import uuid
from typing import Any

from fastapi import FastAPI, Request
from pydantic import BaseModel, Field

from arklex.utils.logging_utils import LogContext

log_context = LogContext(__name__)

app = FastAPI()


class Message(BaseModel):
    role: str = Field(default="")
    content: str = Field(default="")
    tool_calls: list[dict[str, Any]] = Field(default=[])


class ChatChoices(BaseModel):
    message: Message


class ChatCompletion(BaseModel):
    choices: list[ChatChoices]


@app.post("/chat/completions")
async def chat_completions(request: Request) -> ChatCompletion:
    request_body = await request.json()
    # stream = request_body.get("stream", False)

    # Extract the last user message
    user_messages = [
        m for m in request_body.get("messages", []) if m.get("role") == "user"
    ]
    last_user_message = user_messages[-1]["content"]
    system_messages = [
        m for m in request_body.get("messages", []) if m.get("role") == "system"
    ]
    last_system_message = system_messages[-1]["content"] if system_messages else ""
    assistant_messages = [
        m for m in request_body.get("messages", []) if m.get("role") == "assistant"
    ]
    last_assistant_message = (
        assistant_messages[-1]["content"] if assistant_messages else ""
    )
    print(f"Last user message: {last_user_message}")
    print(request_body)
    print(f"Last system message: {last_system_message}")
    print(f"Last assistant message: {last_assistant_message}")

    # Mock intent detection
    if last_user_message.startswith(
        "Given the following intents and their definitions"
    ):
        print("Intent detection request received")
        # Intent detection for test_worker_taskgraph.py
        if last_user_message.endswith("user: How is the weather?\n"):
            return ChatCompletion(
                choices=[
                    ChatChoices(
                        message=Message(
                            tool_calls=[
                                {
                                    "type": "function",
                                    "function": {
                                        "name": "IntentDetectionOutput",
                                        "arguments": '{"intent": "ask about weather"}',
                                    },
                                }
                            ],
                            role="assistant",
                        )
                    )
                ]
            )
        elif last_user_message.endswith("user: Which car would you like to buy?\n"):
            return ChatCompletion(
                choices=[
                    ChatChoices(
                        message=Message(
                            tool_calls=[
                                {
                                    "type": "function",
                                    "function": {
                                        "name": "IntentDetectionOutput",
                                        "arguments": '{"intent": "ask about car options"}',
                                    },
                                }
                            ],
                            role="assistant",
                        )
                    )
                ]
            )
        elif last_user_message.endswith("user: Connect me with a human agent\n"):
            return ChatCompletion(
                choices=[
                    ChatChoices(
                        message=Message(
                            tool_calls=[
                                {
                                    "type": "function",
                                    "function": {
                                        "name": "IntentDetectionOutput",
                                        "arguments": '{"intent": "user want to connect with a human agent"}',
                                    },
                                }
                            ],
                            role="assistant",
                        )
                    )
                ]
            )
        # Intent detection for http_tool_agent_taskgraph.json
        elif last_user_message.endswith(
            "user: What services does your company provide?\n"
        ) or last_user_message.endswith(
            "user: I'm interested in user simulator with a budget of 1000\n"
        ):
            return ChatCompletion(
                choices=[
                    ChatChoices(
                        message=Message(
                            tool_calls=[
                                {
                                    "type": "function",
                                    "function": {
                                        "name": "IntentDetectionOutput",
                                        "arguments": '{"intent": "others"}',
                                    },
                                }
                            ],
                            role="assistant",
                        )
                    )
                ]
            )

        else:
            raise ValueError(f"Unknown intent detection request: {last_user_message}")

    # Mock worker response generation
    elif last_user_message.endswith("assistant:\n"):
        print("Worker response generation request received")
        return ChatCompletion(
            choices=[
                ChatChoices(
                    message=Message(
                        role="assistant", content="This is a demo worker response."
                    )
                )
            ]
        )

    # Mock agent function calling
    elif (
        last_system_message
        == "You are an agent developed by Arklex. Follow these steps carefully:\n\n- Use queryService to get all services provided by Arklex.\n- Use contactTeam to submit service requests."
    ):
        print("Agent function calling request received")
        # Agent function calling for http_tool_agent_taskgraph.json
        if last_user_message == "What services does your company provide?":
            if last_assistant_message != "Calling tool: queryService":
                return ChatCompletion(
                    choices=[
                        ChatChoices(
                            message=Message(
                                role="assistant",
                                content="",
                                tool_calls=[
                                    {
                                        "type": "function",
                                        "function": {
                                            "name": "queryService",
                                            "arguments": "{}",
                                        },
                                        "id": f"call_{str(uuid.uuid4()).replace('-', '')}",
                                    }
                                ],
                            )
                        )
                    ]
                )
            else:
                return ChatCompletion(
                    choices=[
                        ChatChoices(
                            message=Message(
                                role="assistant",
                                content="Arklex provides graph-based chatbots, agents, and user simulators.",
                            )
                        )
                    ]
                )
        elif (
            last_user_message
            == "I'm interested in user simulator with a budget of 1000"
        ):
            if last_assistant_message != "Calling tool: contactTeam":
                return ChatCompletion(
                    choices=[
                        ChatChoices(
                            message=Message(
                                role="assistant",
                                content="",
                                tool_calls=[
                                    {
                                        "type": "function",
                                        "function": {
                                            "name": "contactTeam",
                                            "arguments": '{"ServiceInfo":[{"budget":1000,"service":"user simulator"}]}',
                                        },
                                        "id": f"call_{str(uuid.uuid4()).replace('-', '')}",
                                    }
                                ],
                            )
                        )
                    ]
                )
            else:
                return ChatCompletion(
                    choices=[
                        ChatChoices(
                            message=Message(
                                role="assistant",
                                content="Service request for user simulator with a budget of 1000 submitted successfully.",
                            )
                        )
                    ]
                )

    else:
        raise ValueError(f"Unknown request type: {last_user_message}")


if __name__ == "__main__":
    import uvicorn

    uvicorn.run(
        app,
        host="0.0.0.0",
        port=5005,
    )



# import json
# from typing import Any
# from unittest.mock import Mock, patch

# import requests

# from integration_tests.utils.base import BaseTestOrchestrator, ChatRole


# def create_mock_response(url: str, method: str, **kwargs: dict[str, Any]) -> Mock:
#     mock_response = Mock(spec=requests.Response)
#     mock_response.status_code = 200
#     mock_response.raise_for_status = Mock()

#     request_body = kwargs.get("json", {})

#     if url == "https://api.arklex.test/service":
#         if method == "GET":
#             # Mock response for queryService
#             json_data = {
#                 "services": [
#                     {
#                         "name": "graph-based chatbots",
#                         "description": "Advanced conversational AI using graph structures",
#                         "category": "AI Solutions",
#                     },
#                     {
#                         "name": "agents",
#                         "description": "Intelligent autonomous agents for various tasks",
#                         "category": "AI Solutions",
#                     },
#                     {
#                         "name": "user simulator",
#                         "description": "Simulation tools for user behavior modeling",
#                         "category": "Testing Tools",
#                     },
#                 ]
#             }
#         elif method == "POST":
#             # Mock response for contactTeam
#             json_data = {
#                 "status": "success",
#                 "message": "Service request submitted successfully",
#                 "request_id": "REQ-12345",
#                 "submitted_services": request_body["service"],
#                 "total_budget": sum(item["budget"] for item in request_body["service"]),
#             }
#         else:
#             # Unsupported method
#             json_data = {
#                 "error": "Method Not Allowed",
#                 "message": f"Method {method} not supported",
#             }
#             mock_response.status_code = 405
#     else:
#         # Unknown endpoint
#         json_data = {"error": "Not Found", "message": f"Endpoint {url} not found"}
#         mock_response.status_code = 404

#     mock_response.json.return_value = json_data
#     mock_response.text = json.dumps(json_data)
#     return mock_response


# @patch("arklex.env.tools.custom_tools.http_tool.requests.request")
# def test_http_tool_agent(mock_request: Mock) -> None:
#     orchestrator = BaseTestOrchestrator(
#         "integration_tests/taskgraphs/http_tool_agent_taskgraph.json"
#     )
#     params = BaseTestOrchestrator.init_params()
#     chat_history, params = params["chat_history"], params["parameters"]

#     # start message (direct message)
#     text = "<start>"
#     output = orchestrator.get_response(text, chat_history, params)
#     chat_history.append({"role": ChatRole.USER, "content": text})
#     chat_history.append({"role": ChatRole.ASSISTANT, "content": output["answer"]})
#     params = output["parameters"]
#     assert (
#         output["answer"]
#         == "This is agent developed by Arklex, how can I assist you today?"
#     )

#     # Test case 1: HTTP tool (queryService)
#     mock_request.side_effect = create_mock_response
#     text = "What services does your company provide?"
#     output = orchestrator.get_response(text, chat_history, params)
#     chat_history.append({"role": ChatRole.USER, "content": text})
#     chat_history.append({"role": ChatRole.ASSISTANT, "content": output["answer"]})
#     params = output["parameters"]

#     assert mock_request.call_count == 1
#     get_calls = [
#         call for call in mock_request.call_args_list if call[1].get("method") == "GET"
#     ]
#     assert len(get_calls) == 1
#     get_call = get_calls[0]
#     assert get_call[1]["url"] == "https://api.arklex.test/service"
#     assert get_call[1]["method"] == "GET"
#     assert get_call[1]["headers"]["Authorization"] == "Bearer test-token"
#     assert get_call[1]["headers"]["Content-Type"] == "application/json"

#     response_text = output["answer"].lower()
#     assert "graph-based chatbots" in response_text or "chatbots" in response_text
#     assert "agents" in response_text
#     assert "user simulator" in response_text or "simulator" in response_text

#     # Test Case 2: HTTP tool (contactTeam)
#     mock_request.side_effect = create_mock_response
#     text = "I'm interested in user simulator with a budget of 1000"
#     output = orchestrator.get_response(text, chat_history, params)
#     chat_history.append({"role": ChatRole.USER, "content": text})
#     chat_history.append({"role": ChatRole.ASSISTANT, "content": output["answer"]})
#     params = output["parameters"]

#     post_calls = [
#         call for call in mock_request.call_args_list if call[1].get("method") == "POST"
#     ]
#     assert len(post_calls) >= 1
#     post_call = post_calls[0]
#     assert post_call[1]["url"] == "https://api.arklex.test/service"
#     assert post_call[1]["method"] == "POST"
#     assert post_call[1]["headers"]["Authorization"] == "Bearer test-token"
#     assert post_call[1]["headers"]["Content-Type"] == "application/json"
#     assert post_call[1]["json"] == {
#         "service": [
#             {"service": "user simulator", "budget": 1000},
#         ],
#     }

#     # Verify successful submission response
#     assert (
#         "success" in output["answer"].lower() or "submitted" in output["answer"].lower()
#     )
